@@ -1,4 +1,4 @@
-# $Id: utils.R 1767 2015-09-08 14:44:27Z sgaure $
+# $Id: utils.R 1956 2016-04-08 13:30:28Z sgaure $
 # Author: Simen Gaure
 # Copyright: 2011, Simen Gaure
 # Licence: Artistic 2.0
@@ -128,6 +128,39 @@ diamgraph <- function(flist,approx=TRUE) {
     igraph::diameter(igraph::induced.subgraph(gr,lcl))
 }
 
+
+
+
+
+
+
+#' Find diameters of mobility graphs
+#' 
+#' 'diammatrix' computes the diameters of certain graphs related to convergence
+#' speed of \code{felm}.
+#' 
+#' Each pair of factors (f1,f2) from \code{flist} defines a bipartite graph in
+#' which the vertices are the levels of the factors, and two vertices are
+#' adjacent if they are observed simultaneously. The connected components of
+#' this graph are important for identification of the coefficients for the
+#' factor levels, i.e. for \code{getfe}. But experience and some trials have
+#' led the author to speculate that the diameter of the graph (or its largest
+#' component) is also important for the convergence rate.  Specifically, the
+#' author suspects that under some assumptions, time to convergence goes like
+#' the square of the diameter.  At least in the case of two factors.  This
+#' function computes the diameter for each pair of factors.  If the graph is
+#' disconnected, the largest connected component is used. If \code{accel=TRUE}
+#' (the default), the diameter is approximated from below by drawing two sets
+#' of 10 random vertices and finding the maximum length of the shortest paths
+#' between them.
+#' 
+#' @param flist a list of factors defining the dummies.
+#' @param approx logical. Approximate diameters are computed.
+#' @return A matrix of dimension K x K where K is \code{length(flist)}.
+#' @note This function is not important to the operation of the package, it is
+#' included for easy experimentation with the convergence rate.  It requires
+#' that the suggested package \pkg{igraph} is attached.
+#' @keywords internal
 diammatrix <- function(flist, approx=TRUE) {
 
   flen <- length(flist)
@@ -178,6 +211,30 @@ rankDefic <- function(fl,method='cholesky',mctol=1e-3) {
 # in makeDmatrix/makePmatrix, the weights argument should be the
 # square root of the weights. This is what is stored in internal structures.
 # 
+
+
+
+
+
+
+#' Make sparse matrix of dummies from factor list
+#' 
+#' Given a list of factors, return the matrix of dummies as a sparse matrix.
+#' 
+#' The function returns the model matrix for a list of factors. This matrix is
+#' not used internally by the package, but it's used in some of the
+#' documentation for illustrative purposes.
+#' 
+#' @param fl list of factors.
+#' @param weights numeric vector. Multiplied into the rows.
+#' @return Returns a sparse matrix.
+#' @examples
+#' 
+#'   fl <- lapply(1:3, function(i) factor(sample(3,10,replace=TRUE)))
+#'   fl
+#'   makeDmatrix(fl, weights=seq(0.1,1,0.1))
+#' 
+#' @export makeDmatrix
 makeDmatrix <- function(fl, weights=NULL) {
   # make the D matrix
   # for pure factors f, it's just the t(as(f,'sparseMatrix'))
@@ -271,7 +328,7 @@ swildcard <- function(formula, s=ls(environment(formula)), re=FALSE,
     rchars <- c('*','?')
     wild <- unique(unlist(sapply(rchars,grep, av,fixed=TRUE,value=TRUE)))
     if(length(wild) > 0)
-        rewild <- glob2rx(wild,trim.tail=FALSE)
+        rewild <- utils::glob2rx(wild,trim.tail=FALSE)
     else
         rewild <- NULL
     # quote some regexp-specials
@@ -347,10 +404,24 @@ limlk <- function(mm) {
   min(eigen(mat, only.values=TRUE)$values)
 }
 
-resample <- function(cluster, return.factor=FALSE) {
+resample <- function(cluster, return.factor=FALSE, na.action=NULL, fill=FALSE) {
+  if(is.list(cluster)) {
+    if(is.factor(cluster[[1]])) {
+      if(length(cluster) > 1) warning('List of factors not supported in resample, using first only (',names(cluster)[1],')')
+      cluster <- cluster[[1]]
+    }
+  }
   # resample entire levels of a factor
+  if(length(na.action) > 0) cluster <- factor(cluster[-na.action])
   iclu <- as.integer(cluster)
   cl <- sort(sample(nlevels(cluster), replace=TRUE))
+  if(fill) {
+    tb <- table(cluster)
+    while(sum(tb[cl]) < length(cluster)) {
+      cl <- c(cl,sample(nlevels(cluster),1))
+    }
+    cl <- sort(cl)
+  }
   # find a faster way to do this:
   # s <- sort(unlist(sapply(cl, function(ll) which(clu==ll))))
   s <- NULL
@@ -361,3 +432,88 @@ resample <- function(cluster, return.factor=FALSE) {
   if(return.factor) return(cluster[s])
   sort(s)
 }
+
+getcomp <- function(est, alpha=NULL) {
+  if(nlevels(est$cfactor) == 1) return(list(est=est,alpha=alpha))
+  ok <- which(est$cfactor == 1)
+  res <- est
+  res$cfactor <- factor(res$cfactor[ok])
+  if(!is.null(res$X))
+      res$X <- res$X[ok,]
+  res$residuals <- res$residuals[ok,]
+  res$response <- res$response[ok,]
+  res$fitted.values <- res$fitted.values[ok,]
+  res$r.residuals <- res$r.residuals[ok,]
+  res$fe <- lapply(est$fe, function(f) factor(f[ok]))
+  if(alpha != NULL) {
+    # of the two first factors, remove the components beyond the first
+    # if there are more than two factors, the remaining are assumed to be ok
+    # so find those with 'fe' among the two first, and comp > 1
+    
+    bad <- which((alpha[,'fe'] %in% names(est$fe)[1:2]) & (alpha[,'comp'] > 1))
+    if(length(bad) > 0) alpha <- alpha[-bad,]
+  }
+  return(est=res,alpha=alpha)
+}
+
+#' Chain subset conditions 
+#'
+#' @param ... Logical conditions to be chained.
+#' @param out.vars character. Variables not in data.frame, only needed if you use variables which
+#' are not in the frame.  If \code{out.vars} is not specified, it is assumed to match all variables
+#' starting with a dot ('.').
+#' @return Expression that can be \code{eval}'ed to yield a logical subset mask.
+#' @details
+#' A set of logical conditions are chained, not and'ed. That is, each argument to
+#' \code{chainsubset} is used as a filter to create a smaller dataset. Each subsequent
+#' argument filters further.
+#' For independent conditions this will be the same as and'ing them. I.e.
+#' \code{chainsubset(x < 0 , y < 0)} will yield  the same subset as \code{(x < 0) & (y < 0)}.
+#' However, for e.g. aggregate filters like \code{chainsubset(x < mean(x), y < mean(y))}
+#' we first find all the observations with \code{x < mean(x)}, then among these we
+#' find the ones with \code{y < mean(y)}.  The \code{mean(y)} is now conditional on
+#' \code{x < mean(x)}.
+#' 
+#' @examples
+#' N <- 10000
+#' dat <- data.frame(y=rnorm(N), x=rnorm(N),id=factor(sample(N/100,N,replace=TRUE)))
+#' # It's not the same as and'ing the conditions:
+#' ss <- chainsubset(x < mean(y), y < 3*mean(x))
+#' sum(eval(ss,dat))
+#' sum(evalq(x < mean(y) & y < 3*mean(x), dat))
+#' ss2 <- chainsubset(x < mean(y), y < a*mean(x), out.vars='a')
+#' a <- 3; sum(eval(ss2, dat))
+#' a <- 2; sum(eval(ss2, dat))
+#' # Among observations with x < y, find entire id's with more than
+#' # one fifth of their x's larger than 1/2
+#' ss3 <- chainsubset( x < y, tapply(x,id,function(.xx) {sum(.xx > 1/2) > length(.xx)/5} )[id])
+#'  sum(eval(ss3,dat))
+#' @export
+chainsubset <- function(..., out.vars) {
+  args <- as.list(match.call(expand.dots=TRUE))[-1]
+  args[['out.vars']] <- NULL
+  # We will reduce the arguments. The first argument is
+  # converted to a factor, which is then used to index the variables
+  # in the next element
+  if(length(args) < 1) return(NULL)
+  if(length(args) == 1) return(structure(args[[1]],filter=args))
+  miss <- missing(out.vars)
+  group <- list(as.name('{'))
+  structure(
+      bquote({
+        .first <- (.(args[[1]]))
+        .R <- logical(length(.first))
+        .R[.(Reduce(function(f1,f2) {
+          nm <- all.vars(f2)
+          if(miss)
+              nm <- grep('^\\.',nm,value=TRUE,invert=TRUE)
+          else
+              nm <- nm[!(nm %in% out.vars)]
+          recod <- as.call(c(group,
+                             lapply(nm, function(n) bquote(.(as.name(n)) <- .(as.name(n))[.f, drop=TRUE]))))
+          bquote({.f <- .(f1); local({.(recod); .f[.(f2)]})})
+        }, args[-1],init=quote(which(.first))))] <- TRUE
+        .R
+      }),
+      filter=args)
+}
